# version and release
%define name    cmn
%define version 2004.03.08
%define release 1

# --without cmucl: disable cmucl build
%define cmucl_build %{?_without_cmucl:0}%{!?_without_cmucl:1}

# --without clisp: disable clisp build
%define clisp_build %{?_without_clisp:0}%{!?_without_clisp:1}

# --with sbcl: enable sbcl build
%define sbcl_build  %{?_with_sbcl:1}%{!?_with_sbcl:0}

# lisp versions
%define cmucl_ver  2003.12
%define clisp_ver  2.32
%define sbcl_ver   0.8.6

# various locations
%define cmucl_cmn_bin_dir %{_libdir}/cmucl/lib/cmn
%define cmucl_subsystems  %{_libdir}/cmucl/lib/subsystems
%define clisp_cmn_bin_dir %{_libdir}/clisp/cmn
%define clisp_subsystems  %{_libdir}/clisp/subsystems
%define sbcl_cmn_bin_dir  %{_libdir}/sbcl/cmn
%define cmn_src_dir       %{_datadir}/common-lisp/source/cmn


Summary:      Common Music Notation (CMN %{version})
Name:         %{name}
Version:      %{version}
Release:      %{?release_prefix:%{release_prefix}.}%{release}%{?release_suffix:.%{release_suffix}}
License:      GPL
Group:        Development/Languages
URL:          http://www-ccrma.stanford.edu/software/cmn
Source0:      %{name}.tar.gz
BuildRoot:    %{_tmppath}/%{name}-%{version}-%{release}-root
Packager:     Fernando Lopez-Lezcano
Distribution: Planet CCRMA

Obsoletes:    cmn-source

%description
This package contains CMN [%{version}] (Common Music Notation) 
built on top of cmucl Common Lisp.

Common Music Notation, by Bill Schottstaedt is a Common Lisp program
that can create and display traditional western music scores.

%package   cmucl
Summary:   Common Music Notation binaries for CMUCL (CMN %{version})
Group:     Development/Languages
Requires:  cmucl >= %{cmucl_ver}
Requires:  cmn = %{version}-%{release}
Obsoletes: cmucl-cm-clm-cmn

%description cmucl
This package contains CMN [%{version}] (Common Music Notation) 
built on top of CMUCL Common Lisp.

%package   clisp
Summary:   Common Music Notation binaries for CLISP (CMN %{version})
Group:     Development/Languages
Requires:  clisp >= %{clisp_ver}
Requires:  cmn = %{version}-%{release}
Obsoletes: clisp-cm-clm-cmn

%description clisp
This package contains CMN [%{version}] (Common Music Notation) 
built on top of CLISP Common Lisp.

%package   sbcl
Summary:   Common Music Notation binaries for SBCL (CMN %{version})
Group:     Development/Languages
Requires:  sbcl >= %{sbcl_ver}
Requires:  cmn = %{version}-%{release}

%description sbcl
This package contains CMN [%{version}] (Common Music Notation) 
built on top of SBCL Common Lisp.

%prep
%setup -q -n cmn

%build

%if %{cmucl_build}
mkdir bin-cmucl
# define the cmucl cmn subsystem load program
cat <<EOF >cmn-library.lisp
;;;
;;; CMUCL Common Music Notation (CMN) startup code
;;;
(format t ";;;~%;;; Loading CMN %{version}...~%;;; (please be patient...)~%;;;~%")
(setf *compile-print* nil)
(let ((ext:*gc-verbose* nil)
      (ext:*bytes-consed-between-gcs*
           (* ext:*bytes-consed-between-gcs* 5)))
  (defvar cmn-directory "%{cmn_src_dir}/")
  (defvar cmn-bin-directory "%{cmucl_cmn_bin_dir}/")
  (load "%{cmn_src_dir}/cmn-all.lisp"))
(load-foreign "%{cmucl_cmn_bin_dir}/sgicmn.so")
(format t ";;;~%;;; CMN %{version} loaded~%;;;~%")
EOF

# build cmn
/usr/bin/lisp <<EOF
(setf *compile-print* nil)
(let ((ext:*gc-verbose* nil)
      (ext:*bytes-consed-between-gcs*
           (* ext:*bytes-consed-between-gcs* 5)))
   (in-package :cl-user)
   (defvar cmn-directory "")
   (defvar cmn-bin-directory "bin-cmucl/")
   (load "cmn-all.lisp"))
(compile-file "cmn-library.lisp")
(quit)
EOF
%endif

%if %{clisp_build}
# define the clisp cmn subsystem load program
cat <<EOF >cmn.lisp
;;;
;;; CLISP Common Music Notation (CMN) startup code
;;;
(let* ((custom:*suppress-check-redefinition* t))
   (format t ";;;~%;;; Loading CLM %{version}...~%;;;~%")
   (setf *compile-print* nil)
   (defparameter cmn-directory "%{cmn_src_dir}/")
   (defparameter cmn-bin-directory "%{clisp_cmn_bin_dir}/")
   (load "%{cmn_src_dir}/cmn-all.lisp")
   (format t ";;;~%;;; CMN %{version} loaded~%;;;~%"))
(in-package :cmn)
;;; bring in (quit)
(import 'ext:quit :cmn)
EOF

# build cmn
mkdir bin-clisp
/usr/bin/clisp <<EOF
(setf *compile-print* nil)
(in-package :cl-user)
(defparameter cmn-directory "")
(defparameter cmn-bin-directory "bin-clisp/")
(load "cmn-all.lisp")
(compile-file "cmn.lisp")
(quit)
EOF
%endif

%if %{sbcl_build}
mkdir bin-sbcl
/usr/bin/sbcl <<EOF
EOF
%endif

# build the xcmnw viewer
gcc xcmnw.c -o xcmnw -I/usr/X11R6/include -L/usr/X11R6/lib -lXm -lXp -lXt -lXext -lX11

%install
rm -rf $RPM_BUILD_ROOT

mkdir -p $RPM_BUILD_ROOT%{_bindir}
mkdir -p $RPM_BUILD_ROOT%{cmn_src_dir}
mkdir -p $RPM_BUILD_ROOT%{_docdir}

# copy the cmn original source tree, this is going to be *cmn-source-directory*
# not a good idea to do this if there are patches
(cd $RPM_BUILD_ROOT%{cmn_src_dir}; tar xzf %{SOURCE0}; mv cmn/* .; rmdir cmn)

# CMUCL install
%if %{cmucl_build}
mkdir -p $RPM_BUILD_ROOT%{cmucl_cmn_bin_dir}
mkdir -p $RPM_BUILD_ROOT%{cmucl_subsystems}

(cd %{_builddir}/%{name}/bin-cmucl; tar cf - *)|(cd $RPM_BUILD_ROOT%{cmucl_cmn_bin_dir}; tar xpf -)
cp -p xcmnw $RPM_BUILD_ROOT%{cmucl_cmn_bin_dir}

# install the load subsystem
cp -p cmn-library.x86f $RPM_BUILD_ROOT%{cmucl_subsystems}

# the cmn startup script
cat <<EOF >$RPM_BUILD_ROOT%{_bindir}/cmn-cmucl
#!/bin/bash
# CMN startup script for CMUCL
exec /usr/bin/lisp -load "%{cmucl_subsystems}/cmn-library" -eval "(in-package :cmn)" $@
EOF
chmod 0755 $RPM_BUILD_ROOT%{_bindir}/cmn-cmucl
%endif

# CLISP install
%if %{clisp_build}
mkdir -p $RPM_BUILD_ROOT%{clisp_cmn_bin_dir}
mkdir -p $RPM_BUILD_ROOT%{clisp_subsystems}

# move all the binaries except .lib files (which confuse the clm and cmn requires)
find %{_builddir}/%{name}/bin-clisp -type f ! -name \*.lib -exec mv {} $RPM_BUILD_ROOT%{clisp_cmn_bin_dir} \;

# install the load subsystem
cp -p cmn.fas $RPM_BUILD_ROOT%{clisp_subsystems}/cmn.lib

# the cmn startup script
cat <<EOF >$RPM_BUILD_ROOT%{_bindir}/cmn-clisp
#!/bin/bash
# CMN startup script for clisp
exec /usr/bin/clisp -I -i %{clisp_subsystems}/cmn.lib -p "CMN" $@
EOF
chmod 0755 $RPM_BUILD_ROOT%{_bindir}/cmn-clisp
%endif

# SBCL install
%if %{sbcl_build}
mkdir -p $RPM_BUILD_ROOT%{sbcl_cmn_bin_dir}
%endif

%clean
rm -rf $RPM_BUILD_ROOT

%files
%defattr(-,root,root,-)
%doc
%{cmn_src_dir}

%if %{cmucl_build}
%files cmucl
%defattr(-,root,root,-)
%{cmucl_cmn_bin_dir}
%{cmucl_subsystems}/*
%{_bindir}/*cmucl
%endif

%if %{clisp_build}
%files clisp
%defattr(-,root,root,-)
%{clisp_cmn_bin_dir}
%{clisp_subsystems}/*
%{_bindir}/*clisp
%endif

%if %{sbcl_build}
%files sbcl
%defattr(-,root,root,-)
%{sbcl_cmn_bin_dir}
%{_bindir}/*sbcl
%endif

%changelog
* Mon Mar 15 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.03.08-1
- updated to 2004.03.08 tarfile
* Mon Feb  9 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.02.04-1
- updated to 2004.02.04
* Tue Feb  4 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2003.12.30-1
- clisp: suppress redefinition warnings during load
* Mon Feb  2 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2003.12.30-1
- updated to 2004.01.30
* Wed Jan 28 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2003.12.28-1
- updated to 2004.01.28
* Sat Jan 17 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2003.12.15-1
- initial build
- see old changelog history in the cm package spec file
